#include <windows.h>
#pragma hdrstop
#include "go.h"
#include "boardmgr.h"


/*
 * void InvalidateStone( HWND hwnd, int r, int c)
 *
 * invalidate stone's rect so update will be done at next paint
 */
void InvalidateStone( HWND hwnd, int r, int c)
{
	RECT	rcClientSize;	// size of client area of <hwnd>
	int		x, y;			// window coordinates
	int		iRadiusStone;	// radius of each stone
	int		dx, dy;			// width, height of a cell
	RECT	rcUpdate;		// rectangle to update

	// calculate radius of a stone
	GetClientRect(hwnd, (LPRECT) &rcClientSize);
	dx = rcClientSize.right / (GO_DIMEN + 1) / 2,
	dy = rcClientSize.bottom / (GO_DIMEN + 1) / 2,
	iRadiusStone = min(dx, dy);

	// calculate the position of a stone
	x = (rcClientSize.right * (c + 1)) / (GO_DIMEN + 1);
	y = (rcClientSize.bottom * (r + 1)) / (GO_DIMEN + 1);

	// calculate the rectangle in <hwnd> that needs updating
	rcUpdate.left = x - iRadiusStone;
	rcUpdate.top = y - iRadiusStone;
	rcUpdate.right = x + iRadiusStone;
	rcUpdate.bottom = y + iRadiusStone;

	// let Windows tell AppPaint() to redraw it
	InvalidateRect(hwnd, (LPRECT) &rcUpdate, TRUE);
}


/*
 * void TraverseArmy(PGoBoard pGoBoard, players player, int r, int c, directions dir, void *data)
 *
 * recursivly traverse the army specified by [r][c] and call TraverseFunction
 * for every stone in the army.
 */
void FAR PASCAL _export TraverseArmy(PGoBoard pGoBoard, players player, int r, int c, directions direction, void *data, TraverseFunctionProc action)
{
	// check for invalid board position
	if ((r < 0) || (c < 0) || (r >= GO_DIMEN) || (c >= GO_DIMEN))
		return;

	// check for previous visit
	if ((*pGoBoard)[r][c] < 0)
		return;

	// mark stone as visited
	(*pGoBoard)[r][c] = -(*pGoBoard)[r][c];

	// if stone is current army continue traversal
	if ((*pGoBoard)[r][c] == player)
	{
		// traverse N, S, E, and W if we didn't come from that direction
		if (direction != NORTH)
			TraverseArmy(pGoBoard, player, r - 1, c, SOUTH, data, action);
		if (direction != SOUTH)
			TraverseArmy(pGoBoard, player, r + 1, c, NORTH, data, action);
		if (direction != EAST)
			TraverseArmy(pGoBoard, player, r, c + 1, WEST, data, action);
		if (direction != WEST)
			TraverseArmy(pGoBoard, player, r, c - 1, EAST, data, action);
	}

	// perform action on this stone
	action(pGoBoard, player, r, c, data);

	// unmark stone as visited
	(*pGoBoard)[r][c] = -(*pGoBoard)[r][c];
}


/*
 * void ArmyStrengthFunction(PGoBoard pGoBoard, players player, int r, int c, void *strength)
 *
 */
#pragma argsused
void FAR PASCAL ArmyStrengthFunction(PGoBoard pGoBoard, players player, int r, int c, void *strength)
{
	// check if army is empty and increment strength if it is
	if ((*pGoBoard)[r][c] == EMPTY)
		(*(int *)strength)++;
}


/* int ArmyStrength(PGoBoard pGoBoard, int r, int c)
 *
 * Return the strength of the army at [r][c]
 */
int FAR PASCAL _export ArmyStrength(PGoBoard pGoBoard, int r, int c)
{
	int strength = 0;

	TraverseArmy(pGoBoard, -(*pGoBoard)[r][c], r, c, NULL, &strength, ArmyStrengthFunction);

	return strength;
}


/*
 * void ArmyRemoveFunction(PGoBoard pGoBoard, players player, int r, int c, void *strength)
 *
 */
void FAR PASCAL ArmyRemoveFunction(PGoBoard pGoBoard, players player, int r, int c, void *hwnd)
{
	// delete stone
	if ((*pGoBoard)[r][c] == player)
	{
		(*pGoBoard)[r][c] = EMPTY;
		InvalidateStone(*(HWND *)hwnd, r, c);
	}
}


/* int ArmyRemove(HWND hwnd, PGoBoard pGoBoard, int r, int c)
 *
 * Remove the army at [r][c]
 */
void ArmyRemove(HWND hwnd, PGoBoard pGoBoard, int r, int c)
{
	TraverseArmy(pGoBoard, -(*pGoBoard)[r][c], r, c, NULL, &hwnd, ArmyRemoveFunction);
}


/* Move(HWND hwnd, HANDLE hGoBoard, int r, int c)
 *
 * A move was made at [r,c] by player.  Place stone and remove any captured
 * players from the board.
 */
void FAR PASCAL _export Move(HWND hwnd, HANDLE hGoBoard, int r, int c)
{
	PGoBoard pGoBoard;

	// lock the board
	pGoBoard = (PGoBoard) GlobalLock(hGoBoard);

	// verify that the space is open
	if ((*pGoBoard)[r][c] != EMPTY)
	{
		MessageBeep(MB_ICONASTERISK);
		return;
	}

	// set player
	(*pGoBoard)[r][c] = GetWindowWord(hwnd, GWW_CURPLAYER);

	// invalidate rect around stone
	InvalidateStone(hwnd, r, c);

	// switch players (for remove)
	if (GetWindowWord(hwnd, GWW_CURPLAYER) == WHITE)
		SetWindowWord(hwnd, GWW_CURPLAYER, BLACK);
	else
		SetWindowWord(hwnd, GWW_CURPLAYER, WHITE);

	// if army to north exists, is enemy and is "dead" remove it
	if ((r > 0) && ((*pGoBoard)[r - 1][c] == GetWindowWord(hwnd, GWW_CURPLAYER)) && (ArmyStrength(pGoBoard, r - 1, c) < 1))
		ArmyRemove(hwnd, pGoBoard, r - 1, c);

	// if army to south exists, is CurrentPlayer and is "dead" remove it
	if ((r < (GO_DIMEN-1)) && ((*pGoBoard)[r + 1][c] == GetWindowWord(hwnd, GWW_CURPLAYER)) && (ArmyStrength(pGoBoard, r + 1, c) < 1))
		ArmyRemove(hwnd, pGoBoard, r + 1, c);

	// if army to east exists, is CurrentPlayer and is "dead" remove it
	if ((c < (GO_DIMEN-1)) && ((*pGoBoard)[r][c + 1] == GetWindowWord(hwnd, GWW_CURPLAYER)) && (ArmyStrength(pGoBoard, r, c + 1) < 1))
		ArmyRemove(hwnd, pGoBoard, r, c + 1);

	// if army to west exists, is CurrentPlayer and is "dead" remove it
	if ((c > 0) && ((*pGoBoard)[r][c - 1] == GetWindowWord(hwnd, GWW_CURPLAYER)) && (ArmyStrength(pGoBoard, r, c - 1) < 1))
		ArmyRemove(hwnd, pGoBoard, r, c - 1);

	// if current army is "dead" remove it
	if (ArmyStrength(pGoBoard, r, c) < 1)
		ArmyRemove(hwnd, pGoBoard, r, c);

	// unlock the board
	GlobalUnlock(hGoBoard);
}


/* Pass(HWND)
 *
 * Provided to allow a Go player to pass.
 */
void FAR PASCAL _export Pass(HWND hwnd)
{
	if (GetWindowWord(hwnd, GWW_CURPLAYER) == WHITE)
		SetWindowWord(hwnd, GWW_CURPLAYER, BLACK);
	else
		SetWindowWord(hwnd, GWW_CURPLAYER, WHITE);
}


/* BoardWndProc()
 *
 * The board manager's window proc
 */
LONG FAR PASCAL _export BoardProc(HWND hwnd, unsigned msg, WORD wParam, LONG lParam, HANDLE hGoBoard)
{
	static HANDLE hWhiteLibrary, hBlackLibrary;
	static GoProc WhitePlayerProc, BlackPlayerProc;

	switch (msg)
	{
		case WM_CREATE:
			// set first player
			SetWindowWord(hwnd, GWW_CURPLAYER, WHITE);

			// from dialog, select player modules and initialize them
			hWhiteLibrary = LoadLibrary("MOUSEPLR.DLL");
			{
				WhitePlayerProc = (GoProc)GetProcAddress(hWhiteLibrary, "PlayerProc");
			}
			hBlackLibrary = LoadLibrary("MOUSEPLR.DLL");
			{
				BlackPlayerProc = (GoProc)GetProcAddress(hBlackLibrary, "PlayerProc");
			}

			// pass WM_CREATE on to both players
			(*WhitePlayerProc) (hwnd, msg, WHITE, lParam, hGoBoard);
			(*BlackPlayerProc) (hwnd, msg, BLACK, lParam, hGoBoard);
			return 0L;

		case WM_DESTROY:

			// pass WM_DESTROY on to both players
			(*WhitePlayerProc) (hwnd, msg, wParam, lParam, hGoBoard);
			(*BlackPlayerProc) (hwnd, msg, wParam, lParam, hGoBoard);

			FreeLibrary(hWhiteLibrary);
			FreeLibrary(hBlackLibrary);
			return 0L;

		default:
			// pass message on to current player
			if (GetWindowWord(hwnd, GWW_CURPLAYER) == WHITE)
				return (*WhitePlayerProc) (hwnd, msg, wParam, lParam, hGoBoard);
			else
				return (*BlackPlayerProc) (hwnd, msg, wParam, lParam, hGoBoard);
	}
}
