#include <windows.h>
#pragma hdrstop
#include "about.h"
#include "go.h"
#include "resource.h"


// globals
char	szAppName[] = "Go";	// the name of the application
HANDLE	hInstApp = NULL;	// application instance handle
HWND	hwndApp = NULL;		// application main window
HANDLE	hGoBoard = NULL;	// handle to the Go Board


/* AppPaint(hdc, hGoBoard, prcPaint, ptClientSize)
 *
 * Paint rectangle <*prcPaint> of the Go board resource <hGoBoard> into <hdc>.
 * <ptClient> is the size of the Go board.  It is assumed that <hGoBoard>
 * is already accessed (loaded) if <hGoBoard> is not NULL.
 */
void AppPaint( HDC hdc, HANDLE hGoBoard, LPRECT prcPaint, POINT ptClientSize )
{
	PGoBoard	pGoBoard;		// pointer to GoBoard mem. repr.
	HBRUSH		hbrGoBoard;		// the Go board itself
	HBRUSH		hbrLightLine;	// lines on the Go board
	HBRUSH		hbrDarkLine; 	// shadow lines
	HBRUSH		hbrStone[3]; 	// color of stone for player i (1 or 2)
	HBRUSH		hbrPrev = NULL;	// previously selected brush
	int 		r, c;			// row, column index
	int 		x, y;			// window coordinates
	int 		xMin, xMax;		// left-, right-most line positions
	int 		yMin, yMax;		// top-, bottom-most line position
	int 		iCell;			// cell contents (0=empty, 1,2=stone)
	int 		iRadiusSpot; 	// radius of each of the 9 spots
	int 		iRadiusStone;	// radius of each stone
	int 		dx, dy;

	// lock the resource (unless there isn't one)
	if (hGoBoard != NULL)
		pGoBoard = (PGoBoard) GlobalLock(hGoBoard);

	// create brushes for drawing Go board and stones
	if (((hbrGoBoard = CreateSolidBrush(rgbGOBOARD)) == NULL) ||
		((hbrLightLine = CreateSolidBrush(rgbGOLIGHTLINE)) == NULL) ||
		((hbrDarkLine = CreateSolidBrush(rgbGODARKLINE)) == NULL) ||
		((hbrStone[1] = CreateSolidBrush(rgbGOWHITE)) == NULL) ||
		((hbrStone[2] = CreateSolidBrush(rgbGOBLACK)) == NULL) )
		goto AppPaint_EXIT;

	// calculate extent of lines
	xMin = ptClientSize.x / (GO_DIMEN + 1);
	xMax = (ptClientSize.x * GO_DIMEN) / (GO_DIMEN + 1);
	yMin = ptClientSize.y / (GO_DIMEN + 1);
	yMax = (ptClientSize.y * GO_DIMEN) / (GO_DIMEN + 1);

	// calculate radius of stones
	dx = ptClientSize.x / (GO_DIMEN + 1) / 2,
	dy = ptClientSize.y / (GO_DIMEN + 1) / 2,
	iRadiusStone = min(dx, dy);

	// calculate radius of spots
	dx = ptClientSize.x / SPOT_SHRINK;
	dy = ptClientSize.y / SPOT_SHRINK;
	iRadiusSpot = min(dx, dy);

	// draw Go board itself
	hbrPrev = SelectObject(hdc, hbrGoBoard);
	PatBlt(hdc, 0, 0, ptClientSize.x, ptClientSize.y, PATCOPY);

	/* if <hGoBoard> is null (i.e. there is no current resource) or if
	 * the medLock() failed, just leave the Go board drawn without
	 * lines or stones as an indication to the user
	 */
	if ((hGoBoard == NULL) || (pGoBoard == NULL))
		goto AppPaint_EXIT;

	// draw dark lines
	SelectObject(hdc, hbrDarkLine);
	for (r = 0; r < GO_DIMEN; r++)		// draw horizontal lines
		PatBlt(hdc, xMin - SHWD, (ptClientSize.y * (r + 1)) / (GO_DIMEN + 1) - SHWD, xMax - xMin + SHWD + 1, SHWD, PATCOPY);
	for (c = 0; c < GO_DIMEN; c++)		// draw vertical lines
		PatBlt(hdc, (ptClientSize.x * (c + 1)) / (GO_DIMEN + 1) - SHWD, yMin - SHWD, SHWD, yMax - yMin + SHWD + 1, PATCOPY);

	// draw light lines
	SelectObject(hdc, hbrLightLine);
	for (r = 0; r < GO_DIMEN; r++)		// draw horizontal lines
		PatBlt(hdc, xMin, (ptClientSize.y * (r + 1)) / (GO_DIMEN + 1), xMax - xMin + 1, 1, PATCOPY);
	for (c = 0; c < GO_DIMEN; c++)		// draw vertical lines
		PatBlt(hdc, (ptClientSize.x * (c + 1)) / (GO_DIMEN + 1), yMin, 1, yMax - yMin + 1, PATCOPY);

	// draw the 9 spots
	SelectObject(hdc, hbrDarkLine);
	for (r = 3; r < GO_DIMEN; r += 6)
	{
		y = (ptClientSize.y * (r + 1)) / (GO_DIMEN + 1);
		for (c = 3; c < GO_DIMEN; c += 6)
		{
			x = (ptClientSize.x * (c + 1)) / (GO_DIMEN + 1);

			// draw spot centered at (x,y)
			PatBlt(hdc, x - iRadiusSpot, y - iRadiusSpot, 2 * iRadiusSpot, 2 * iRadiusSpot, PATCOPY);
		}
	}

	// draw stones
	for (r = 0; r < GO_DIMEN; r++)
	{
		y = (ptClientSize.y * (r + 1)) / (GO_DIMEN + 1);

		// speed optimization: don't paint rows unnecessarily
		if ((y + iRadiusStone < prcPaint->top) || (y - iRadiusStone > prcPaint->bottom))
			continue;

		for (c = 0; c < GO_DIMEN; c++)
		{
			x = (ptClientSize.x * (c + 1)) / (GO_DIMEN + 1);

			// draw stone centered at (x,y)
			if ((iCell = (*pGoBoard)[r][c]) != 0)
			{
				SelectObject(hdc, hbrStone[iCell]);
				Ellipse(hdc, x - iRadiusStone, y - iRadiusStone, x + iRadiusStone, y + iRadiusStone);
			}
		}
	}

AppPaint_EXIT:

	// select the original brush before deleting any brush
	if (hbrPrev != NULL)
		SelectObject(hdc, hbrPrev);

	// delete brushes
	if (hbrGoBoard != NULL)
		DeleteObject(hbrGoBoard);
	if (hbrLightLine != NULL)
		DeleteObject(hbrLightLine);
	if (hbrDarkLine != NULL)
		DeleteObject(hbrDarkLine);
	if (hbrStone[1] != NULL)
		DeleteObject(hbrStone[1]);
	if (hbrStone[2] != NULL)
		DeleteObject(hbrStone[2]);

	// unlock the resource
	if (hGoBoard != NULL)
		GlobalUnlock(hGoBoard);
}


/* GoWndProc()
 *
 * The window procedure for the main application window.
 */
LONG FAR PASCAL GoWndProc(HWND hwnd, unsigned msg, WORD wParam, LONG lParam)
{
	static GoProc BoardProc = NULL;
	static HANDLE hLibrary;
	PAINTSTRUCT	ps;
	RECT		rc;
	POINT		pt;

	switch (msg)
	{
		case WM_CREATE:

			// load GoBoardManager DLL
			hLibrary = LoadLibrary("BOARDMGR.DLL");
			{
				BoardProc = (GoProc)GetProcAddress(hLibrary, "BoardProc");
			}

			// allocate memory for Go board
			hGoBoard = GlobalAlloc(GMEM_MOVEABLE|GMEM_ZEROINIT, sizeof(GoBoard));

			// pass WM_CREATE on to BoardManager
			return BoardProc(hwnd, msg, wParam, lParam, hGoBoard);

		case WM_ERASEBKGND:

			// no need to erase background -- we are painting it all
			return 0L;

		case WM_PAINT:

			BeginPaint(hwnd, &ps);
			GetClientRect(hwnd, &rc);
			pt.x = rc.right;
			pt.y = rc.bottom;
			AppPaint(ps.hdc, hGoBoard, &ps.rcPaint, pt);
			EndPaint(hwnd, &ps);
			return 0L;

		case WM_COMMAND:

			switch (wParam)
			{
				case IDM_HELPABOUT:

					// display the "About" box
					DoAbout(hwnd, hInstApp);
					break;
			}
			return 0L;

		case WM_DESTROY:

			// pass WM_DESTROY on to BoardManager
			BoardProc(hwnd, msg, wParam, lParam, hGoBoard);
			FreeLibrary(hLibrary);
			BoardProc = NULL;

			// free board global memory
			hGoBoard = GlobalFree(hGoBoard);
			PostQuitMessage(0);

		default:

			// pass message on to BoardManager if it exists
			if (BoardProc)
				return BoardProc(hwnd, msg, wParam, lParam, hGoBoard);
			else
				return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}


/* WinMain()
 *
 * Application entry point.
 */
#pragma argsused
int PASCAL WinMain(HANDLE hInst, HANDLE hPrev, LPSTR szCmdLine, int sw)
{
	MSG     	msg;		// a message from GetMessage()
	WNDCLASS	cls;		// the Go application window class

	// save instance handle for dialog boxes
	hInstApp = hInst;

	if (hPrev == NULL)
	{
		// register application display class
		cls.hCursor        = LoadCursor(NULL, IDC_ARROW);
		cls.hIcon          = LoadIcon(hInst, szAppName);
		cls.lpszMenuName   = szAppName;
		cls.lpszClassName  = szAppName;
		cls.hbrBackground  = GetStockObject(LTGRAY_BRUSH);
		cls.hInstance      = hInst;
		cls.style          = CS_VREDRAW | CS_HREDRAW;
		cls.lpfnWndProc    = GoWndProc;
		cls.cbWndExtra     = sizeof(WORD) + (2 * sizeof(LONG));
		cls.cbClsExtra     = 0;

		if (!RegisterClass(&cls))
			return FALSE;
	}

	// create the application main window
	hwndApp = CreateWindow(szAppName,	// Class name
			szAppName,			// Caption
			WS_OVERLAPPEDWINDOW,	// Style bits
			CW_USEDEFAULT, CW_USEDEFAULT, // Position
			CW_USEDEFAULT, CW_USEDEFAULT, // Size
			(HWND) NULL,		// Parent window
			(HMENU) NULL,		// use class menu
			(HANDLE) hInst,		// instance handle
			(LPSTR) NULL		// no params to pass on
		);

	ShowWindow(hwndApp, sw);
	UpdateWindow(hwndApp);

	// poll for messages
	while (GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
}
