#include <windows.h>
#pragma hdrstop
#include "go.h"
#include "mouseplr.h"
#include "boardmgr.h"


/* AppClick(hwnd, hGoBoard, ptClick, player)
 *
 * The user clicked at point <ptClick> in window <hwnd>, that's currently
 * displaying Go board <hGoBoard>.  It is assumed that <hGoBoard>
 * is valid if it is not NULL.
 */
void AppClick( HWND hwnd, HANDLE hGoBoard, POINT ptClick)
{
	PGoBoard  pGoBoard;      // pointer to GoBoard mem. repr.
	int       r, c;          // row, column index
	int       xCell, yCell;  // size of a Go board cell
	RECT      rcClientSize;  // size of client area of <hwnd>

	// if no go board, don't process clicks
	if (hGoBoard == NULL)
		return;

	// set (r,c) to the row/column that was clicked
	GetClientRect(hwnd, &rcClientSize);
	xCell = rcClientSize.right / (GO_DIMEN + 1);
	yCell = rcClientSize.bottom / (GO_DIMEN + 1);
	r = (ptClick.y + yCell / 2) * (GO_DIMEN + 1) / rcClientSize.bottom - 1;
	r = (r < 0 ? 0 : (r >= GO_DIMEN ? (GO_DIMEN - 1) : r));
	c = (ptClick.x + xCell / 2) * (GO_DIMEN + 1) / rcClientSize.right - 1;
	c = (c < 0 ? 0 : (c >= GO_DIMEN ? (GO_DIMEN - 1) : c));

	// lock the board
	pGoBoard = (PGoBoard) GlobalLock(hGoBoard);

	// verify that the space is open and set player
	if ((*pGoBoard)[r][c] != EMPTY)
	{
		MessageBeep(MB_ICONASTERISK);
	}
	else
	{
		// place stone and update the go board by removing any captured armies
		MoveProc Move = (MoveProc)GetWindowLong(hwnd, GWL_MOVEPROC);
		Move(hwnd, hGoBoard, r, c);
	}

	// unlock the board
	GlobalUnlock(hGoBoard);
}


/* PlayerProc()
 *
 * The window procedure for the mouse player
 */
LONG FAR PASCAL _export PlayerProc(HWND hwnd, unsigned msg, WORD wParam, LONG lParam, HANDLE hGoBoard)
{
	static HANDLE hLibrary;

	switch (msg)
	{
		case WM_CREATE:

			// load GoBoardManager DLL
			hLibrary = LoadLibrary("BOARDMGR.DLL");
			{
				SetWindowLong(hwnd, GWL_MOVEPROC, (LONG)GetProcAddress(hLibrary, "Move"));
				SetWindowLong(hwnd, GWL_PASSPROC, (LONG)GetProcAddress(hLibrary, "Pass"));
			}
			return 0L;

		case WM_LBUTTONDOWN:

			AppClick(hwnd, hGoBoard, MAKEPOINT(lParam));
			return 0L;

		case WM_RBUTTONDOWN:
		{
			PassProc Pass= (PassProc)GetWindowLong(hwnd, GWL_PASSPROC);
			Pass(hwnd);
			return 0L;
		}

		case WM_DESTROY:

			FreeLibrary(hLibrary);
			return 0L;

		default:

			return DefWindowProc(hwnd, msg, wParam, lParam);
	}
}
